@@ -129,7 +129,6 @@ sudo_term_noecho_v1(int fd)
 {
     debug_decl(sudo_term_noecho, SUDO_DEBUG_UTIL)
 
-again:
     if (!changed && tcgetattr(fd, &oterm) != 0)
 	debug_return_bool(false);
     (void) memcpy(&term, &oterm, sizeof(term));
@@ -141,11 +140,6 @@ again:
 	changed = 1;
 	debug_return_bool(true);
     }
-    if (got_sigttou) {
-	/* We were in the background, so oterm is probably bogus. */
-	kill(getpid(), SIGTTOU);
-	goto again;
-    }
     debug_return_bool(false);
 }
 
@@ -159,9 +153,8 @@ sudo_term_raw_v1(int fd, int isig)
     struct termios term;
     debug_decl(sudo_term_raw, SUDO_DEBUG_UTIL)
 
-again:
     if (!changed && tcgetattr(fd, &oterm) != 0)
-	return 0;
+	debug_return_bool(false);
     (void) memcpy(&term, &oterm, sizeof(term));
     /* Set terminal to raw mode */
     term.c_cc[VMIN] = 1;
@@ -175,11 +168,6 @@ again:
 	changed = 1;
     	debug_return_bool(true);
     }
-    if (got_sigttou) {
-	/* We were in the background, so oterm is probably bogus. */
-	kill(getpid(), SIGTTOU);
-	goto again;
-    }
     debug_return_bool(false);
 }
 
@@ -192,9 +180,8 @@ sudo_term_cbreak_v1(int fd)
 {
     debug_decl(sudo_term_cbreak, SUDO_DEBUG_UTIL)
 
-again:
     if (!changed && tcgetattr(fd, &oterm) != 0)
-	return 0;
+	debug_return_bool(false);
     (void) memcpy(&term, &oterm, sizeof(term));
     /* Set terminal to half-cooked mode */
     term.c_cc[VMIN] = 1;
@@ -212,11 +199,6 @@ again:
 	changed = 1;
 	debug_return_bool(true);
     }
-    if (got_sigttou) {
-	/* We were in the background, so oterm is probably bogus. */
-	kill(getpid(), SIGTTOU);
-	goto again;
-    }
     debug_return_bool(false);
 }
 
@@ -230,15 +212,9 @@ sudo_term_copy_v1(int src, int dst)
     struct termios tt;
     debug_decl(sudo_term_copy, SUDO_DEBUG_UTIL)
 
-again:
     if (tcgetattr(src, &tt) != 0)
 	debug_return_bool(false);
     if (tcsetattr_nobg(dst, TCSASOFT|TCSAFLUSH, &tt) == 0)
 	debug_return_bool(true);
-    if (got_sigttou) {
-	/* We were in the background, so tt is probably bogus. */
-	kill(getpid(), SIGTTOU);
-	goto again;
-    }
     debug_return_bool(false);
 }
