@@ -46,14 +46,6 @@
 # endif /* __hpux */
 # include <prot.h>
 #endif /* HAVE_GETPRPWNAM */
-#ifdef HAVE_GETPWANAM
-# include <sys/label.h>
-# include <sys/audit.h>
-# include <pwdadj.h>
-#endif /* HAVE_GETPWANAM */
-#ifdef HAVE_GETAUTHUID
-# include <auth.h>
-#endif /* HAVE_GETAUTHUID */
 
 #include "sudoers.h"
 
@@ -79,11 +71,15 @@ sudo_getepw(const struct passwd *pw)
     if (!iscomsec())
 	goto done;
 #endif /* HAVE_ISCOMSEC */
-#ifdef HAVE_ISSECURE
-    if (!issecure())
-	goto done;
-#endif /* HAVE_ISSECURE */
 
+#ifdef HAVE_GETPWNAM_SHADOW
+    {
+	struct passwd *spw;
+
+	if ((spw = getpwnam_shadow(pw->pw_name)) != NULL)
+	    epw = spw->pw_passwd;
+    }
+#endif /* HAVE_GETPWNAM_SHADOW */
 #ifdef HAVE_GETPRPWNAM
     {
 	struct pr_passwd *spw;
@@ -104,32 +100,8 @@ sudo_getepw(const struct passwd *pw)
 	    epw = spw->sp_pwdp;
     }
 #endif /* HAVE_GETSPNAM */
-#ifdef HAVE_GETSPWUID
-    {
-	struct s_passwd *spw;
 
-	if ((spw = getspwuid(pw->pw_uid)) && spw->pw_passwd)
-	    epw = spw->pw_passwd;
-    }
-#endif /* HAVE_GETSPWUID */
-#ifdef HAVE_GETPWANAM
-    {
-	struct passwd_adjunct *spw;
-
-	if ((spw = getpwanam(pw->pw_name)) && spw->pwa_passwd)
-	    epw = spw->pwa_passwd;
-    }
-#endif /* HAVE_GETPWANAM */
-#ifdef HAVE_GETAUTHUID
-    {
-	AUTHORIZATION *spw;
-
-	if ((spw = getauthuid(pw->pw_uid)) && spw->a_password)
-	    epw = spw->a_password;
-    }
-#endif /* HAVE_GETAUTHUID */
-
-#if defined(HAVE_ISCOMSEC) || defined(HAVE_ISSECURE)
+#if defined(HAVE_ISCOMSEC)
 done:
 #endif
     /* If no shadow password, fall back on regular password. */
@@ -146,15 +118,6 @@ sudo_setspent(void)
 #endif
 #ifdef HAVE_GETSPNAM
     setspent();
-#endif
-#ifdef HAVE_GETSPWUID
-    setspwent();
-#endif
-#ifdef HAVE_GETPWANAM
-    setpwaent();
-#endif
-#ifdef HAVE_GETAUTHUID
-    setauthent();
 #endif
     debug_return;
 }
@@ -169,15 +132,6 @@ sudo_endspent(void)
 #endif
 #ifdef HAVE_GETSPNAM
     endspent();
-#endif
-#ifdef HAVE_GETSPWUID
-    endspwent();
-#endif
-#ifdef HAVE_GETPWANAM
-    endpwaent();
-#endif
-#ifdef HAVE_GETAUTHUID
-    endauthent();
 #endif
     debug_return;
 }
