@@ -18,18 +18,8 @@
 
 #include <sys/types.h>
 #include <stdio.h>
-#ifdef STDC_HEADERS
-# include <stdlib.h>
-# include <stddef.h>
-#else
-# ifdef HAVE_STDLIB_H
-#  include <stdlib.h>
-# endif
-#endif /* STDC_HEADERS */
+#include <stdlib.h>
 #ifdef HAVE_STRING_H
-# if defined(HAVE_MEMORY_H) && !defined(STDC_HEADERS)
-#  include <memory.h>
-# endif
 # include <string.h>
 #endif /* HAVE_STRING_H */
 #ifdef HAVE_STRINGS_H
@@ -70,8 +60,11 @@ main(int argc, char *argv[])
     symbols_file = argv[2];
 
     handle = sudo_dso_load(plugin_path, SUDO_DSO_LAZY|SUDO_DSO_GLOBAL);
-    if (handle == NULL)
-	sudo_fatalx_nodebug("unable to load %s: %s", plugin_path, sudo_dso_strerror());
+    if (handle == NULL) {
+	const char *errstr = sudo_dso_strerror();
+	sudo_fatalx_nodebug("unable to load %s: %s", plugin_path,
+	    errstr ? errstr : "unknown error");
+    }
 
     fp = fopen(symbols_file, "r");
     if (fp == NULL)
@@ -83,8 +76,9 @@ main(int argc, char *argv[])
 	    *cp = '\0';
 	sym = sudo_dso_findsym(handle, line);
 	if (sym == NULL) {
+	    const char *errstr = sudo_dso_strerror();
 	    printf("%s: test %d: unable to resolve symbol %s: %s\n",
-		getprogname(), ntests, line, sudo_dso_strerror());
+		getprogname(), ntests, line, errstr ? errstr : "unknown error");
 	    errors++;
 	}
     }
